
module ice_temp_mod                 ! mis en module le 13 mars 2007 (Cat)

! la production de chaleur est mise dans une routine du module

use module3d_phy
use tridiagmod     ! module dans lequel est la routine tridiag
use module_choix


implicit none

integer :: iec,jec,ngr
integer :: nfracq   ! exposant fracq

integer :: iq       ! choix du type de routine chaleur

real :: sx,sy,sx2,sy2,deh22,tss
real ::  dou,dah,duu,dzz,dzi,chalbed,ct_bas,ct_haut
real :: acof1,bcof1,ccof1,s0mer ! pour la temprature de l'eau de mer
                                !  la base de l'ice shelf
real :: advecx,advecy

!integer :: ifail             ! permet de detecter les erreurs 
real :: chalk_1               ! utilise pour le calcul de chalk : glace pose
real :: chalk_2               ! utilise pour le calcul de chalk : ice streams et ice shelves
real :: chalbed_1             ! utilise pour le calcul de chalbed
real :: coefadv               ! pour limiter le flux de chaleur horiz.

real :: ecart_phid            ! pour limiter l'impact de la chaleur de glissement si base froide
                              ! ancienne valeur 0.5. 

! tableaux 1D
! _______________
real,dimension(nzm+1) :: abis,bbis,cbis,rbis,hbis

real,dimension(nn) :: aa=0.              ! work arrays for tridiag
real,dimension(nn) :: bb=0.              ! work arrays for tridiag
real,dimension(nn) :: cc=1.              ! work arrays for tridiag
real,dimension(nn) :: rr=0.              ! work arrays for tridiag
real,dimension(nn) :: hh=0.              ! work arrays for tridiag


! tableaux de travail 2D 
! ___________________________
real, dimension(nx,ny):: tbmer   ! temperature de la mer a la base de l'ice shelf
real,dimension(nx,ny) :: alpha   ! pente locale sur noeud majeur
real,dimension(nx,ny) :: ubar    ! vitesse sur noeud majeur

real,dimension(nx,ny) :: chalglissx,chalglissy ! chaleur de glissement



! Tableaux de travail 3D
! __________________________

real, dimension(nx,ny,nz,n1poly:n2poly) :: chalx, chaly
real, dimension(nx,ny,nz) :: chal2_x, chal2_y, chal2_z, chal2_xy
! utilis pour calculer la chaleur de deformation selon xx yy zz et xy
real, dimension(nx,ny,n1poly:n2poly) :: ffx,ffy
real, dimension(nx,ny,nz+nzm) :: T3D_new




contains
!------------------------------------------------------------------------------------------
subroutine init_icetemp

!     variables dependant du pas de temps dtt

  ctm=dtt*cm/rom/cpm/dzm/dzm
  dttdx=dtt/dx


!________________________________________________________________
! La temperature du point de congelation de l'eau de mer
! est tiree de Jenkins (1991), Tb est en degres C
! _______________________________________________________________

  acof1 = - 0.0575
  bcof1 = 0.0901
  ccof1 = 7.61E-4
  s0mer = 34.75


!!     nouvelle formulation de fracq
!  nfracq=15  ! ancienne formulation
nfracq=1
fracq=(1.-(1.-de/2.)**nfracq)/nfracq

write(num_rep_42,*) 'calcul des temperatures'
write(num_rep_42,*) 'nfracq=',nfracq

iq=4


if (iq.eq.1) write(num_rep_42,*) 'chaleur demi maille, iq=',iq
if (iq.eq.2) write(num_rep_42,*) 'chaleur centree, iq=',iq
if (iq.eq.3) write(num_rep_42,*) 'chaleur pente, iq=',iq
if (iq.eq.4) write(num_rep_42,*) 'chaleur u_taub, iq=',iq
if (iq.eq.5) write(num_rep_42,*) 'chaleur u_taub_stag, iq=',iq
if (iq.eq.6) write(num_rep_42,*) 'chaleur Q_SIA_stag,iq=',iq
if (iq.eq.7) write(num_rep_42,*) 'chaleur Q_all_stag,iq=',iq
write(num_rep_42,*)

ecart_phid=0.5
write(num_rep_42,*) ' si base froide, la chaleur de glissement est en exp(delta T * ecart_phid)'
write(num_rep_42,*) 'ecart_phid=',ecart_phid

write(num_rep_42,*) '---------------------------------------------------------------------------'

end subroutine init_icetemp


!------------------------------------------------------------------------------------------
subroutine icetemp()

  if (itracebug.eq.1)  call tracebug(' Entree dans routine icetemp')

  !________________________________________________________________
  ! La temperature du point de congelation de l'eau de mer
  ! est tiree de Jenkins (1991), Tb est en degres C
  ! _______________________________________________________________

  do j=1,ny
     do i=1,nx
        if (flot(i,j)) &
             tbmer(i,j) = acof1*s0mer + bcof1 + ccof1*H(i,j)*ro/row
     end do
  end do


  ! Appel aux proprietes thermiques de la glace : conductivite, capacite claorifique -tpmp

  call thermal_prop




  if (iq.eq.1) call  Q_prod_demi
  if (iq.eq.2) call  Q_prod_centre
  if (iq.eq.3) call  Q_prod_pente
  if (iq.eq.4) call  Q_U_taub
  if (iq.eq.5) call  Q_U_taub_stag
  if (iq.eq.6) call  Q_SIA_stag
  if (iq.eq.7) call  Q_all_stag

  !     dans le socle : les lments de la matrice tridiag sont invariants dans l'espace
  do k=nz+1,nz+nzm-1
     aa(k)=-ctm
     bb(k)=1.+2.*ctm
     cc(k)=-ctm
  end do

  !     conditions aux limites pour le socle
  aa(nz+nzm)=-1.
  bb(nz+nzm)=1.
  cc(nz+nzm)=0.


  !     boucle de resolution des temperatures colonne par colonne
  do j=2,ny-1
     do i=2,nx-1
        rr(nz+nzm)=-dzm*ghf(i,j)/cm  ! dpend de i,j pour un flux gothermique variable
        !      -----------------------------cas general (H>10m)

        if (H(i,j).gt.10.) then

           T(i,j,1)=min(0.,Ts(i,j))

           aa(1)=0.
           bb(1)=1.
           cc(1)=0.
           rr(1)=T(i,j,1)


           !        variables de calcul dans la glace

           deh22=2.*de*h(i,j)**2
           dou=dtt/de/de/h(i,j)/h(i,j)
           dah=dtt/h(i,j)/de
           duu=dtt/2./de
           ct_bas=2.*(Ct(i,j,1)*Ct(i,j,2))/(Ct(i,j,1)+Ct(i,j,2))


           !        indice upwind pour l'advection horizontale
           !        Le test se fait maintenant sur les vitesses


           !           ecoulement vers l'OUEST
           if ((uxbar(i+1,j).lt.0.).and.(uxbar(i,j).le.0.)) then
              i1=i+1
              i2=i
              iec=0

              !           ecoulement vers l'EST
           else if ((uxbar(i+1,j).ge.0.).and.(uxbar(i,j).gt.0.)) then
              i1=i
              i2=i-1
              iec=0
              !           ecoulement  convergent
           else if ((uxbar(i+1,j).lt.0.).and.(uxbar(i,j).gt.0.)) then
              i1=i+1
              i2=i-1
              iec=1
              !           ecoulement divergent
           else if ((uxbar(i+1,j).ge.0.).and.(uxbar(i,j).le.0.)) then
              i1=i
              i2=i
              iec=-1
           endif

           !           ecoulement vers lE SUD
           if ((uybar(i,j+1).lt.0.).and.(uybar(i,j).le.0.)) then
              j1=j+1
              j2=j
              jec=0
              !           ecoulement vers lE NORD
           else if ((uybar(i,j+1).ge.0.).and.(uybar(i,j).gt.0.)) then
              j1=j
              j2=j-1
              jec=0
              !           ecoulement  convergent
           else if ((uybar(i,j+1).lt.0.).and.(uybar(i,j).gt.0.)) then
              j1=j+1
              j2=j-1
              jec=1
              !           ecoulement  divergent
           else if ((uybar(i,j+1).ge.0.).and.(uybar(i,j).le.0.)) then
              j1=j
              j2=j
              jec=-1
           endif


           do k=2,nz-1


              dzz=dou/cp(i,j,k)

              ! conductivit au milieu des mailles
              Ct_haut=ct_bas
              Ct_bas=2.*(Ct(i,j,k)*Ct(i,j,k+1))/(Ct(i,j,k)+Ct(i,j,k+1))

              !         advection verticale centree
              aa(k)=-dzz*ct_haut-uzr(i,j,k)*dah/2.
              bb(k)=1.+dzz*(ct_haut+ct_bas)
              cc(k)=-dzz*ct_bas+uzr(i,j,k)*dah/2.

              !         avection selon x
              !         ----------------
              if (iec.eq.0) then

                 !    coefadv limite l'advection afin de ne pas amener plus de
                 !    chaleur qu'il n'y en a (limite a delta T/2)
                 !  Attention : avril 2010 :
                 !  on enleve cette limitation qui a l'air de modifier sensiblement les resultats 
                 ! (comparaison avec Hassine)

                 !                 coefadv=max(1.,abs(uxbar(I1,J)))
                 !                 coefadv=min(0.5/dttdx/coefadv,1.)
                 coefadv=1.                                    !correction adv avril 2010

                 advecx = coefadv*(UX(I1,J,K)*(T(I1,J,K)-T(I2,J,K)))

              else
                 !                 coefadv=max(1.,abs(uxbar(i,j)))
                 !                 coefadv=min(0.5/dttdx/coefadv,1.)
                 coefadv=1.                                    !correction adv avril 2010
                 advecx=coefadv*ux(i,j,k)*(T(i,j,k)-T(i-1,j,k))
                 coefadv=max(1.,abs(uxbar(i+1,j)))
                 coefadv=min(0.5/dttdx/coefadv,1.)
                 coefadv=1.                                    !correction adv avril 2010
                 advecx=iec*(advecx+ &
                      coefadv*ux(i+1,j,k)*(T(i+1,j,k)-T(i,j,k)))
              endif

              !         avection selon y
              !         ----------------
              if (jec.eq.0) then
                 !                 coefadv=max(1.,abs(uybar(i,j1)))
                 !                 coefadv=min(0.5/dttdx/coefadv,1.)
                 coefadv=1.                                    !correction adv avril 2010
                 advecy = coefadv*uy(i,j1,k)*(T(i,j1,k)-T(i,j2,k))

              else
                 !                 coefadv=max(1.,abs(uybar(i,j)))
                 !                 coefadv=min(0.5/dttdx/coefadv,1.)
                 coefadv=1.                                    !correction adv avril 2010
                 advecy=coefadv*uy(i,j,k)*(T(i,j,k)-T(i,j-1,k))
                 coefadv=max(1.,abs(uybar(i+1,j)))
                 coefadv=min(0.5/dttdx/coefadv,1.)
                 advecy=jec*(advecy+ &
                      (uy(i,j+1,k)*(T(i,j+1,k)-T(i,j,k))))
                 advecy=jec*((uy(i,j,k)*(T(i,j,k)-T(i,j-1,k)))+ &
                      coefadv*uy(i,j+1,k)*(T(i,j+1,k)-T(i,j,k)))
              endif

              rr(k)=T(i,j,k)+chaldef_maj(i,j,k)*dtt
              rr(k)=rr(k)-dttdx*(advecx+advecy)


           end do
           !        fin boucle k, ecriture matrice tridiag dans la glace





           ! Conditions aux limites  la base de la glace
           !________________________________________________

           !           base froide 
           if ( ((ibase(i,j).eq.1).or.(ibase(i,j).eq.4) &
                .or. ((ibase(i,j).eq.5).and.(T(i,j,nz).lt.TPMP(i,j,nz)))) &
                .and..not.flot(i,j)) then

              ibase(i,j)=1
              bb(nz)=1.

              if (ncond.eq.1) then ! avec socle
                 dzi=h(i,j)*de*cm/ct(i,j,nz)
                 aa(nz)=-dzm/(dzm+dzi)
                 cc(nz)=-dzi/(dzm+dzi)
                 rr(nz)=h(i,j)*de*phid(i,j)*dzm/ct(i,j,nz)/(dzm+dzi)

              else                 ! sans socle
                 aa(nz)=-1.
                 cc(nz)=0.
                 rr(nz)=-(ghf(i,j)-phid(i,j))/ct(i,j,nz)*h(i,j)*de
              endif

           else

              !          base temperee ou Shelf
              !          ----------------------               
              if (ibase(i,j).eq.5)  ibase(i,j)=2
              ibase(i,j)=max(ibase(i,j),2)
              aa(nz)=0.
              bb(nz)=1.
              cc(nz)=0.

              if (.not.flot(i,j)) then
                 rr(nz)=TPMP(i,j,nz)
              else
                 rr(nz)=Tbmer(i,j)
              end if

           endif

           if (ncond.eq.1) then ! avec socle
              do k=nz+1,nz+nzm-1
                 rr(k)=T(i,j,k)
              end do
              call TRIDIAG (aa,bb,cc,rr,hh,nz+nzm,ifail)

           else                 ! sans socle


              call TRIDIAG (AA,BB,CC,RR,HH,NZ,iFAIL)
           endif

           ! test permettant d'ecrire les variables quand il y a une erreur
           ! iFAIL = 1 : erreur dans tridiag

           if (ifail.eq.1) then
              write(6,*)'erreur dans TRIDIAG'
              write(6,*)'i = ',i,' j = ',j
              write(6,*) 'a, b, c, r, u'
              do k=1,nz+nzm
                 write(6,*)'k = ',k
                 write(6,*) aa(k),bb(k),cc(k),rr(k),hh(k)
              end do
              stop
           end if

           !         temperature dans le socle si ncond=0, lineaire avec le gradient ghf
           if (ncond.eq.0) then
              do k=nz+1,nz+nzm
                 hh(k)=hh(nz)-dzm*(k-nz)*ghf(i,j)/cm
              end do
           endif



           do k=1,nz+nzm
              T3D_new(i,j,k)=hh(k)
           end do
           tbdot(i,j)=(T3D_new(i,j,nz)-t(i,j,nz))/dtt


           !      ------------------------------------------------- glace tres fine (H<10m ou H=0)
        else if (H(i,j).le.10.) then

           !       Pour eviter des problemes lorsque H est inferieur a 10 m
           !       profil lineaire, pente=flux geothermique
           !       Le cas sans glace est traite de la meme facon  (DOU=0)

           ! ........................................ avec calcul dans le socle
           if (NCOND.eq.1.AND..NOT.FLOT(I,J)) then

              if (H(I,J).gt.0.) then
                 !           gradient dans le socle
                 DOU=(T(I,J,NZ+1)-T(I,J,NZ))/DZM*CM
                 DOU=DOU/CT(I,J,NZ)*DE*H(I,J)

                 TSS=min(0.,TS(I,J))
                 do K=1,NZ
                    T3D_new(I,J,K)=TSS+DOU*(K-1.)
                 end do

                 !         pas de glace
              else
                 TSS=Ts(I,J)
                 do K=1,NZ
                    T3D_new(I,J,K)=TSS
                 end do
              endif

              !         calcul dans le socle meme s'il n'y a pas de glace
              RR(NZ)=T3D_new(I,J,NZ)
              AA(NZ)=0.
              BB(NZ)=1.
              CC(NZ)=0.

              do K=NZ+1,NZ+NZM-1
                 RR(K)=T(I,J,K)
              end do

              !         creation de nouveaux tableaux juste pour tridiag      
              do k=1,NZM+1
                 ABIS(k)=AA(NZ-1+k)
                 BBIS(k)=BB(NZ-1+k)
                 CBIS(k)=CC(NZ-1+k)
                 RBIS(k)=RR(NZ-1+k)
              end do

              call TRIDIAG (ABIS,BBIS,CBIS,RBIS,HBIS,NZM+1,iFAIL)
              ! test permettant d'ecrire les variables quand il y a une erreur
              ! iFAIL = 1 : erreur dans tridiag
              IF (iFAIL.EQ.1) THEN
                 write(6,*)'Erreur dans TRIDIAG'
                 write(6,*)'I = ',I,' J = ',J
                 write(6,*) 'A, B, C, R, U'
                 do k=1,NZ+NZM
                    write(6,*)'k = ',k
                    write(6,*) ABIS(k),BBIS(k),CBIS(k),RBIS(k),HBIS(k)
                 end do
                 STOP
              END IF

              do k=1,NZM+1
                 HH(NZ-1+k)=HBIS(k)
              end do

              ! ........................................ sans calcul dans le socle
           else
              !                        calotte posee                    
              if ((H(I,J).gt.0.).AND..NOT.FLOT(I,J)) then
                 DOU=-GHF(i,j)/CT(I,J,NZ)*DE*H(I,J)
                 TSS=min(0.,TS(I,J))
                 do K=1,NZ
                    T3D_new(I,J,K)=TSS+DOU*(K-1.)
                 end do

                 !                        shelf
              else if ((H(I,J).gt.0.).AND.FLOT(I,J)) then
                 TSS=min(0.,TS(I,J))
                 DOU=(TBMER(I,J)-TSS)*DE
                 do K=1,NZ
                    T3D_new(I,J,K)=TSS+DOU*(K-1.)
                 end do

                 !                       pas de glace
              else
                 TSS=Ts(I,J)
                 do K=1,NZ
                    T3D_new(I,J,K)=TSS
                 end do
              endif

              !         temperature dans le socle, lineaire avec le gradient ghf

              IF (.NOT.FLOT(I,J)) THEN
                 do K=NZ+1,NZ+NZM
                    HH(K)=T3D_new(I,J,NZ)-DZM*(K-NZ)*GHF(i,j)/CM
                 end do

              ELSE

                 DO K=NZ+1,NZ+NZM
                    HH(K)=Tbmer(i,j)-DZM*(K-NZ)*GHF(i,j)/CM
                 END DO
              ENDIF


           endif
           !         fin du test socle

           do K=NZ+1,NZ+NZM
              T3D_new(I,J,K)=HH(K)
           end do


           TBDOT(I,J)=(T3D_new(I,J,NZ)-T(I,J,NZ))/DTT
           BMELT(I,J)=0.
           IBASE(I,J)=5
           phid(i,j)=0.

        endif
        !       fin du test sur l'epaisseur de glace

     end do
  end do


  do j=1,ny
     do i=1,nx
        H1(I,J)=H(I,J)
        B1(I,J)=B(I,J)
        TPMP(I,J,1)=0.
     end do
  end do


  !     attribution finale de la temperature
  ! limitation  3deg de variation par dtt et pas plus froid que -70

  do k=1,nz+nzm
     do j=1,ny
        do i=1,nx
           if (H(i,j).gt.10.) then

              Tdot(k)=T3D_new(i,j,k)-T(i,j,k)

              if ((k.gt.1).and.(Tdot(k).lt.-3.)) Tdot(k)=-3.
              if ((k.gt.1).and.(Tdot(k).gt.3.)) Tdot(k)=3.

              T(i,j,k)=T(i,j,k)+Tdot(k)

              if (T(i,j,k).lt.-70.) then
                 T(i,j,k)=-70.
              endif
           endif

        end do
     end do
  end do

  do k=2,nz
     do j=1,ny
        do i=1,nx
           if (T(i,j,k).gt.TPMP(i,j,k)) then
              T(i,j,k)=TPMP(i,j,k)
              ibase(i,j)=2
           endif
        end do
     end do
  end do

  !     temperature sur les bords :
  !     -------------------------


  do k=1,nz
     do j=1,ny
        !        T(nx,j,k)= T(nx-1,j,k)
        !        T(1,j,k)= T(2,j,k) 
        T(nx,j,k)= Ts(nx,j)
        T(1,j,k)= Ts(1,j)
     end do
     !
     do i=1,nx
        !        T(i,1,k)=T(i,2,k) 
        !        T(i,ny,k)=T(i,ny-1,k) 
        T(i,1,k)=Ts(i,1)
        T(i,ny,k)=Ts(i,ny)
     end do
  end do


  do k=1,nz
     do j=1,ny
        do i=1,nx
           if (H(i,j).le.1.) T(i,j,k)=Ts(i,j)
        end do
     end do
  end do


  return
end subroutine icetemp
!-------------------------------------------------------------------------------------
! calcul avec l'ancienne mthode sur les demi-mailles

subroutine Q_prod_demi

  do k=2,nz
     do j=2,ny-1
        do i=2,nx-1


! calcul de la chaleur de deformation selon xx yy zz et xy
! pour les ice-streams et ice-shelves
! les divers eps sont calculs dans strain-rate pour l'ensemble de la grille

        if ((flot(i,j).or.flgzmx(i,j).or.flgzmx(i+1,j)).or.  &
                        (flgzmy(i,j).or.flgzmy(i,j+1))) then

              chal2_x(i,j,k)=2.*visc(i,j,k)*epsxx(i,j)**2
              chal2_y(i,j,k)=2.*visc(i,j,k)*epsyy(i,j)**2
              chal2_z(i,j,k)=2.*visc(i,j,k)*(-epsxx(i,j)-epsyy(i,j))**2

!  epsxy est calcule sur les noeuds mineur 1/2,1/2, faire la moyenne
              chal2_xy(i,j,k)=(epsxy(i,j)+epsxy(i+1,j)+epsxy(i+1,j+1)+epsxy(i,j+1))
              chal2_xy(i,j,k)=2.*visc(i,j,k)*(chal2_xy(i,j,k)*0.25)**2

        else   ! glace pose
              chal2_x(i,j,k)=0.
              chal2_y(i,j,k)=0.
              chal2_z(i,j,k)=0.
              chal2_xy(i,j,k)=0.
        endif
        end do
     end do
  end do

!  partie SIA   calcul de la chaleur produite sur chaque demi maille
  do l=n1poly,n2poly
     do j=2,ny
        do i=2,nx

!          ffx a 3 dimensions !
           ffx(i,j,l)=ddx(i,j,l)*sdx(i,j)*sdx(i,j)
           ffy(i,j,l)=ddy(i,j,l)*sdy(i,j)*sdy(i,j)
        end do
     end do
  end do



  do l=n1poly,n2poly
     do k=2,nz
        do j=2,ny
           do i=2,nx
              if ((.not.flotmx(i,j)).and.(.not.gzmx(i,j))) then
                 chalx(i,j,k,l)=(btt(i-1,j,k,l)+btt(i,j,k,l))*ffx(i,j,l) !&
!                      *ro*g*e(k)**(glen(l)+1)/cp(i,j,k)

              else if (gzmx(i,j)) then ! ice streams
                  chalx(i,j,k,l)=0.

              else                     ! ice shelves  
                  chalx(i,j,k,l)=0.

              endif

              if ((.not.flotmy(i,j)).and.(.not.gzmy(i,j))) then
                 chaly(i,j,k,l)=(btt(i,j-1,k,l)+btt(i,j,k,l))*ffy(i,j,l) !&
!                      *ro*g*e(k)**(glen(l)+1)/cp(i,j,k)

              else if (gzmy(i,j)) then  ! ice streams
                  chaly(i,j,k,l)=0.

              else                      ! ice shelves
                  chaly(i,j,k,l)=0.
              endif

           end do
        end do
     end do
  end do


 


!         nouvelle formulation de chaldef_maj(i,j,k), le 4 vient des moyennes
!         Btt et gauche et droite (ou haut et bas) mais il faut sommer
!         les productions x et y
!         ancienne formulation CHAL=(RO*G*H(I,J))**4*(SX2+SY2)*(SX*SX+SY*SY) 

do k=2,nz
   do j=1,ny-1
      do i=1,nx-1

! modif christophe mars 2000 : chalx et chaly sont a 4 dim
              chaldef_maj(i,j,k)= 0.

! chalk_2 pour ice shelves et ice streams
              chalk_2=(chal2_x(i,j,k)+chal2_y(i,j,k) + &
                   chal2_z(i,j,k)+chal2_xy(i,j,k))/cp(i,j,k)

! chalk_1 pour la partie pose
              do l=n1poly,n2poly

! on somme la chaleur due aux diverses lois de dformation et celles en x et y
                 chalk_1=(chalx(i,j,k,l)+chalx(i+1,j,k,l))+ &
                      (chaly(i,j,k,l)+chaly(i,j+1,k,l))

                 chalk_1=ro*g*chalk_1/4.*(e(k)**(glen(l)+1))/cp(i,j,k)
                 chaldef_maj(i,j,k)= chalk_1 + chaldef_maj(i,j,k) 
              enddo

! Pour shelves et streams,  on ajoute chalk_2
                chaldef_maj(i,j,k) = chaldef_maj(i,j,k) + chalk_2
             end do
          end do
       end do

! chaleur produite a la base par le glissement

  do j=2,ny
     do i=2,nx
        
        if (gzmx(i,j)) then
           chalglissx(i,j)= abs(uxbar(i,j)*tobmx(i,j))
        else
           chalglissx(i,j)=ddbx(i,j)*sdx(i,j)**2*ro*g*Hmx(i,j)
        endif

        if (gzmy(i,j)) then
           chalglissy(i,j)= abs(uybar(i,j)*tobmy(i,j))
        else
           chalglissy(i,j)=ddby(i,j)*sdy(i,j)**2*ro*g*Hmy(i,j)
        endif

     end do
  end do


!  BOUNDARY CONDITION ICE-ROCK interface 

k=nz


do j=2,ny-1
   do i=2,nx-1

! l'ancien chalbed correspond a chaldef_maj(i,j,nz)  : pas la peine de recalculer
!----------------------------------------------------
!!$      chalbed=0.
!!$      do l=n1poly,n2poly
!!$         chalbed_1=(chalx(i,j,k,l)+chalx(i+1,j,k,l)) &
!!$              + (chaly(i,j,k,l)+chaly(i,j+1,k,l))
!!$         chalbed_1=ro*g*chalbed_1/4.*h(i,j)
!!$         chalbed= chalbed_1 +chalbed
!!$      enddo


!        rajouter un flux de chaleur pour la production par deformation
!        dans la derniere 1/2 maille et par le glissement
!        attention phid est >0 et ghf est <0
           if (icouple.ge.1.and..not.flot(i,j)) then

!           phid avec fonte sous les streams
!              phid(i,j)=0.25*(chalglissx(i,j)+chalglissx(i+1,j)+ &
!                        chalglissy(i,j)+chalglissy(i,j+1)) + &
!                        chalbed*fracq


! moyenne phid sur 4 points : attention 0.5 pour moyenne gauche - droite 
! mais les chaleurs x,y s'ajoutent

!              phid(i,j)=0.5*(chalglissx(i,j)+chalglissx(i+1,j)+ &
!                        chalglissy(i,j)+chalglissy(i,j+1)) 

! moyenne phid sur 12 points. On moyenne chaque chaleur et on somme x et y

             phid(i,j)=0.25*(chalglissx(i,j)+chalglissx(i+1,j)+ &
                        chalglissy(i,j)+chalglissy(i,j+1)) 
             phid(i,j)=phid(i,j)+0.125*( &
                  ((chalglissx(i,j-1)+chalglissx(i+1,j-1))+ &
                  (chalglissx(i,j+1)+chalglissx(i+1,j+1))) + &
                  ((chalglissy(i-1,j)+chalglissy(i-1,j+1)) +  &
                  (chalglissy(i+1,j)+chalglissy(i+1,j+1))))
                        
! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
              phid(i,j)=phid(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)

! pour sorties Heino
              chalgliss_maj(i,j)=phid(i,j)

              phid(i,j)=phid(i,j)+ chaldef_maj(i,j,nz)*fracq*H(i,j)*cp(i,j,nz)


           else
              phid(i,j)=0.
           endif
        end do
     end do
return
end subroutine Q_prod_demi

!-------------------------------------------------------------------------------------

! calcul avec la somme des carres

subroutine Q_prod_centre

  do k=2,nz  
     do j=2,ny-1
        do i=2,nx-1


! calcul de la chaleur de deformation selon xx yy zz et xy
! pour les ice-streams et ice-shelves
! les divers eps sont calculs dans strain-rate pour l'ensemble de la grille

        if ((flot(i,j).or.flgzmx(i,j).or.flgzmx(i+1,j)).or.  &
                        (flgzmy(i,j).or.flgzmy(i,j+1))) then

              chal2_x(i,j,k)=2.*visc(i,j,k)*epsxx(i,j)**2
              chal2_y(i,j,k)=2.*visc(i,j,k)*epsyy(i,j)**2
              chal2_z(i,j,k)=2.*visc(i,j,k)*(-epsxx(i,j)-epsyy(i,j))**2

!  epsxy est calcule sur les noeuds mineur 1/2,1/2, faire la moyenne
              chal2_xy(i,j,k)=(epsxy(i,j)+epsxy(i+1,j)+epsxy(i+1,j+1)+epsxy(i,j+1))
              chal2_xy(i,j,k)=2.*visc(i,j,k)*(chal2_xy(i,j,k)*0.25)**2

        else   ! glace pose
              chal2_x(i,j,k)=0.
              chal2_y(i,j,k)=0.
              chal2_z(i,j,k)=0.
              chal2_xy(i,j,k)=0.
        endif
        end do
     end do
  end do

!  partie SIA   calcul de la chaleur produite sur chaque demi maille
  do l=n1poly,n2poly
     do j=2,ny
        do i=2,nx

!          ffx a 3 dimensions !
           ffx(i,j,l)=ddx(i,j,l)*sdx(i,j)*sdx(i,j)
           ffy(i,j,l)=ddy(i,j,l)*sdy(i,j)*sdy(i,j)
        end do
     end do
  end do



  do l=n1poly,n2poly
     do k=2,nz
        do j=2,ny
           do i=2,nx
              if ((.not.flotmx(i,j)).and.(.not.gzmx(i,j))) then
                 chalx(i,j,k,l)=(btt(i-1,j,k,l)+btt(i,j,k,l))*ffx(i,j,l) !&
!                      *ro*g*e(k)**(glen(l)+1)/cp(i,j,k)

              else if (gzmx(i,j)) then ! ice streams
                  chalx(i,j,k,l)=0.

              else                     ! ice shelves  
                  chalx(i,j,k,l)=0.

              endif

              if ((.not.flotmy(i,j)).and.(.not.gzmy(i,j))) then
                 chaly(i,j,k,l)=(btt(i,j-1,k,l)+btt(i,j,k,l))*ffy(i,j,l) !&
!                      *ro*g*e(k)**(glen(l)+1)/cp(i,j,k)

              else if (gzmy(i,j)) then  ! ice streams
                  chaly(i,j,k,l)=0.

              else                      ! ice shelves
                  chaly(i,j,k,l)=0.
              endif

           end do
        end do
     end do
  end do


 


!         nouvelle formulation de chaldef_maj(i,j,k), le 4 vient des moyennes
!         Btt et gauche et droite (ou haut et bas) mais il faut sommer
!         les productions x et y
!         ancienne formulation CHAL=(RO*G*H(I,J))**4*(SX2+SY2)*(SX*SX+SY*SY) 

do k=2,nz
   do j=1,ny-1
      do i=1,nx-1

! modif christophe mars 2000 : chalx et chaly sont a 4 dim
              chaldef_maj(i,j,k)= 0.

! chalk_2 pour ice shelves et ice streams
              chalk_2=(chal2_x(i,j,k)+chal2_y(i,j,k) + &
                   chal2_z(i,j,k)+chal2_xy(i,j,k))/cp(i,j,k)

! chalk_1 pour la partie pose
              do l=n1poly,n2poly

! on somme la chaleur due aux diverses lois de dformation et celles en x et y
! la somme se fait par (Cx2+Cy2)** 0.5. Le 4 vient des moyennes Btt + des moyennes gauche-droite


                 chalk_1=(chalx(i,j,k,l)+chalx(i+1,j,k,l))**2+ &
                      (chaly(i,j,k,l)+chaly(i,j+1,k,l))**2
                 chalk_1=chalk_1**0.5

                 chalk_1=ro*g*chalk_1/4.*(e(k)**(glen(l)+1))/cp(i,j,k)
                 chaldef_maj(i,j,k)= chalk_1 + chaldef_maj(i,j,k) 
              enddo

! Pour shelves et streams,  on ajoute chalk_2
                chaldef_maj(i,j,k) = chaldef_maj(i,j,k) + chalk_2
             end do
          end do
       end do

! chaleur produite a la base par le glissement

  do j=2,ny
     do i=2,nx
        
        if (gzmx(i,j)) then
           chalglissx(i,j)= abs(uxbar(i,j)*tobmx(i,j))
        else
           chalglissx(i,j)=ddbx(i,j)*sdx(i,j)**2*ro*g*Hmx(i,j)
        endif

        if (gzmy(i,j)) then
           chalglissy(i,j)= abs(uybar(i,j)*tobmy(i,j))
        else
           chalglissy(i,j)=ddby(i,j)*sdy(i,j)**2*ro*g*Hmy(i,j)
        endif

     end do
  end do


!  BOUNDARY CONDITION ICE-ROCK interface 

k=nz


do j=2,ny-1
   do i=2,nx-1



!        rajouter un flux de chaleur pour la production par deformation
!        dans la derniere 1/2 maille et par le glissement
!        attention phid est >0 et ghf est <0
           if (icouple.ge.1.and..not.flot(i,j)) then

!           phid avec fonte sous les streams
!              phid(i,j)=0.25*(chalglissx(i,j)+chalglissx(i+1,j)+ &
!                        chalglissy(i,j)+chalglissy(i,j+1)) + &
!                        chalbed*fracq


! moyenne phid sur 4 points formulation (A2+B2)**0.5
!
              chalgliss_maj(i,j)=(chalglissx(i,j)+chalglissx(i+1,j))**2+  &
                   (chalglissy(i,j)+chalglissy(i,j+1))**2

              chalgliss_maj(i,j)=chalgliss_maj(i,j)**0.5

              chalgliss_maj(i,j)=chalgliss_maj(i,j)*0.5 ! les moyennes droite gauche

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
              chalgliss_maj(i,j)=chalgliss_maj(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)


! flux total a rajouter  la base
              phid(i,j)=chalgliss_maj(i,j)+chaldef_maj(i,j,nz)*fracq*H(i,j)*cp(i,j,nz)


           else
              phid(i,j)=0.
           endif
        end do
     end do
return
end subroutine Q_prod_centre

!-------------------------------------------------------------------------------------


! calcul pour essayer d'avoir la chaleur en alpha4

subroutine Q_prod_pente

implicit none
real, dimension(nx,ny) :: pente2_maj   ! pente au carre sur les noeuds majeurs

do j=2,ny
   do i=2,nx
      pente2_maj(i,j)=(sdx(i,j)+sdx(i+1,j))**2 + &
                      (sdy(i,j)+sdy(i,j+1))**2

      pente2_maj(i,j)=pente2_maj(i,j)*0.25    !pour la moyenne sur sdx et sdy
   end do
end do

  do k=2,nz  
     do j=2,ny-1
        do i=2,nx-1


! calcul de la chaleur de deformation selon xx yy zz et xy
! pour les ice-streams et ice-shelves
! les divers eps sont calculs dans strain-rate pour l'ensemble de la grille

        if ((flot(i,j).or.flgzmx(i,j).or.flgzmx(i+1,j)).or.  &
                        (flgzmy(i,j).or.flgzmy(i,j+1))) then

              chal2_x(i,j,k)=2.*visc(i,j,k)*epsxx(i,j)**2
              chal2_y(i,j,k)=2.*visc(i,j,k)*epsyy(i,j)**2
              chal2_z(i,j,k)=2.*visc(i,j,k)*(-epsxx(i,j)-epsyy(i,j))**2

!  epsxy est calcule sur les noeuds mineur 1/2,1/2, faire la moyenne
              chal2_xy(i,j,k)=(epsxy(i,j)+epsxy(i+1,j)+epsxy(i+1,j+1)+epsxy(i,j+1))
              chal2_xy(i,j,k)=2.*visc(i,j,k)*(chal2_xy(i,j,k)*0.25)**2

        else   ! glace pose
              chal2_x(i,j,k)=0.
              chal2_y(i,j,k)=0.
              chal2_z(i,j,k)=0.
              chal2_xy(i,j,k)=0.
        endif
        end do
     end do
  end do

!  partie SIA   calcul de la chaleur produite sur chaque demi maille
!!$  do l=n1poly,n2poly
!!$     do j=2,ny
!!$        do i=2,nx
!!$
!!$!          ffx a 3 dimensions !  
!!$           ffx(i,j,l)=ddx(i,j,l)*sdx(i,j)*sdx(i,j)
!!$           ffy(i,j,l)=ddy(i,j,l)*sdy(i,j)*sdy(i,j)
!!$        end do
!!$     end do
!!$  end do



  do l=n1poly,n2poly
     do k=2,nz
        do j=2,ny
           do i=2,nx
              if ((.not.flotmx(i,j)).and.(.not.gzmx(i,j))) then
                 chalx(i,j,k,l)=(btt(i-1,j,k,l)+btt(i,j,k,l))*ddx(i,j,l)

              else if (gzmx(i,j)) then ! ice streams
                  chalx(i,j,k,l)=0.

              else                     ! ice shelves  
                  chalx(i,j,k,l)=0.

              endif

              if ((.not.flotmy(i,j)).and.(.not.gzmy(i,j))) then
                 chaly(i,j,k,l)=(btt(i,j-1,k,l)+btt(i,j,k,l))*ddy(i,j,l)

              else if (gzmy(i,j)) then  ! ice streams
                  chaly(i,j,k,l)=0.

              else                      ! ice shelves
                  chaly(i,j,k,l)=0.
              endif

           end do
        end do
     end do
  end do


 


!         nouvelle formulation de chaldef_maj(i,j,k), le 4 vient des moyennes
!         Btt et gauche et droite (ou haut et bas) mais il faut sommer
!         les productions x et y
!         ancienne formulation CHAL=(RO*G*H(I,J))**4*(SX2+SY2)*(SX*SX+SY*SY) 

do k=2,nz
   do j=1,ny-1
      do i=1,nx-1

! modif christophe mars 2000 : chalx et chaly sont a 4 dim
              chaldef_maj(i,j,k)= 0.

! chalk_2 pour ice shelves et ice streams
              chalk_2=(chal2_x(i,j,k)+chal2_y(i,j,k) + &
                   chal2_z(i,j,k)+chal2_xy(i,j,k))/cp(i,j,k)

! chalk_1 pour la partie pose
              do l=n1poly,n2poly

! on somme la chaleur due aux diverses lois de dformation et celles en x et y
! la somme se fait par (Cx2+Cy2)** 0.5. Le 4 vient des moyennes Btt + des moyennes gauche-droite

! on fait la moyenne des termes ddx*btt (*0.25 pour cette moyenne, le 0.5 est pour les btt)
                 chalk_1=(chalx(i,j,k,l)+chalx(i+1,j,k,l))+ &
                      (chaly(i,j,k,l)+chaly(i,j+1,k,l))

                 chalk_1=chalk_1*0.25*0.5

! on multiplie par la pente moyenne au carre sur le noeud majeur
                 chalk_1=chalk_1*pente2_maj(i,j)

                 chalk_1=ro*g*chalk_1*(e(k)**(glen(l)+1))/cp(i,j,k)  ! attention plus de /4
                 chaldef_maj(i,j,k)= chalk_1 + chaldef_maj(i,j,k) 
              enddo

! Pour shelves et streams,  on ajoute chalk_2
                chaldef_maj(i,j,k) = chaldef_maj(i,j,k) + chalk_2
             end do
          end do
       end do

! chaleur produite a la base par le glissement

  do j=2,ny
     do i=2,nx
        
        if (gzmx(i,j)) then
           chalglissx(i,j)= abs(uxbar(i,j)*tobmx(i,j))
        else
           chalglissx(i,j)=ddbx(i,j)*sdx(i,j)**2*ro*g*Hmx(i,j)
        endif

        if (gzmy(i,j)) then
           chalglissy(i,j)= abs(uybar(i,j)*tobmy(i,j))
        else
           chalglissy(i,j)=ddby(i,j)*sdy(i,j)**2*ro*g*Hmy(i,j)
        endif

     end do
  end do


!  BOUNDARY CONDITION ICE-ROCK interface 

k=nz


do j=2,ny-1
   do i=2,nx-1



!        rajouter un flux de chaleur pour la production par deformation
!        dans la derniere 1/2 maille et par le glissement
!        attention phid est >0 et ghf est <0
           if (icouple.ge.1.and..not.flot(i,j)) then

!           phid avec fonte sous les streams
!              phid(i,j)=0.25*(chalglissx(i,j)+chalglissx(i+1,j)+ &
!                        chalglissy(i,j)+chalglissy(i,j+1)) + &
!                        chalbed*fracq


! moyenne phid sur 4 points formulation (A2+B2)**0.5  ! je garde pour l'instant a revoir
!
              chalgliss_maj(i,j)=(chalglissx(i,j)+chalglissx(i+1,j))**2+  &
                   (chalglissy(i,j)+chalglissy(i,j+1))**2

              chalgliss_maj(i,j)=chalgliss_maj(i,j)**0.5

              chalgliss_maj(i,j)=chalgliss_maj(i,j)*0.5 ! les moyennes droite gauche

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
              chalgliss_maj(i,j)=chalgliss_maj(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)


! flux total a rajouter  la base

              phid(i,j)=chalgliss_maj(i,j)+chaldef_maj(i,j,nz)*fracq*H(i,j)*cp(i,j,nz)


           else
              phid(i,j)=0.
           endif
        end do
     end do
return
end subroutine Q_prod_pente

!-------------------------------------------------------------------------------------

! routine qui prend la chaleur remise au socle

subroutine Q_U_taub

implicit none
real, dimension(nx,ny) :: pente_maj   ! pente au carre sur les noeuds majeurs
real,dimension(nx,ny) :: vit_maj      ! Ubar moyennee sur les noeuds majeurs
real,dimension(nx,ny) :: uslid_maj      ! Uslid moyennee sur les noeuds majeurs

do j=2,ny
   do i=2,nx


      pente_maj(i,j)=(sdx(i,j)+sdx(i+1,j))**2 + &          ! pente
                      (sdy(i,j)+sdy(i,j+1))**2

      pente_maj(i,j)=pente_maj(i,j)*0.25    ! 0.25pour la moyenne sur sdx et sdy
      pente_maj(i,j)=pente_maj(i,j)**0.5

      vit_maj(i,j)=(uxbar(i,j)+uxbar(i+1,j))**2 + &        ! vitesse de bilan
                      (uybar(i,j)+uybar(i,j+1))**2
      vit_maj(i,j)=vit_maj(i,j)*0.25
      vit_maj(i,j)=vit_maj(i,j)**0.5

      uslid_maj(i,j)=(ux(i,j,nz)+ux(i+1,j,nz))**2 + &        ! vitesse de bilan
                      (uy(i,j,nz)+uy(i,j+1,nz))**2
      uslid_maj(i,j)=uslid_maj(i,j)*0.25
      uslid_maj(i,j)=uslid_maj(i,j)**0.5

   end do
end do



chaldef_maj(:,:,:)=0.
chalgliss_maj(:,:)=ro*g*H(:,:)*pente_maj(:,:)*uslid_maj(:,:)
chaldef_maj(:,:,nz)=ro*g*H(:,:)*pente_maj(:,:)*vit_maj(:,:)-chalgliss_maj(:,:)

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte

if (ispinup.eq.0) then
   chalgliss_maj(:,:)=chalgliss_maj(:,:)*exp((T(:,:,nz)-tpmp(:,:,nz))*ecart_phid)
end if

! flux total a rajouter  la base
phid(:,:)=chaldef_maj(:,:,nz)+chalgliss_maj(:,:)


if (ispinup.eq.3) then
   phid(:,:)=ro*g*H(:,:)*pente_maj(:,:)*vit_maj(:,:)
   chaldef_maj(:,:,nz)=0
end if


return
end subroutine Q_U_taub

!----------------------------------------------------------------------------------
! routine qui prend la chaleur remise au socle mais sur les mailles staggered
! les noeuds stag sont ceux au milieux des mailles vitesses

subroutine Q_U_taub_stag

implicit none
real, dimension(nx,ny) :: pente_stag   ! pente au carre sur les noeuds stag
real,dimension(nx,ny) :: vit_stag      ! Ubar moyennee sur les noeuds stag
real,dimension(nx,ny) :: uslid_stag    ! Uslid moyennee sur les noeuds stag
real,dimension(nx,ny) :: Hmxy          ! Epaisseur moyenne sur les noeuds stag
real,dimension(nx,ny) :: Qslid         ! chaleur produite par glissement sur les noeuds stag
real,dimension(nx,ny) :: Qdef          ! chaleur produite par deformation sur les noeuds stag


do j=2,ny
   do i=2,nx


      pente_stag(i,j)=(sdx(i,j)+sdx(i,j-1))**2 + &          ! pente
                      (sdy(i,j)+sdy(i-1,j))**2

      pente_stag(i,j)=pente_stag(i,j)*0.25    ! 0.25pour la moyenne sur sdx et sdy
      pente_stag(i,j)=pente_stag(i,j)**0.5

      vit_stag(i,j)=(uxbar(i,j)+uxbar(i,j-1))**2 + &        ! vitesse de bilan
                      (uybar(i,j)+uybar(i-1,j))**2
      vit_stag(i,j)=vit_stag(i,j)*0.25
      vit_stag(i,j)=vit_stag(i,j)**0.5

      uslid_stag(i,j)=(ux(i,j,nz)+ux(i,j-1,nz))**2 + &        ! vitesse de bilan
                      (uy(i,j,nz)+uy(i-1,j,nz))**2
      uslid_stag(i,j)=uslid_stag(i,j)*0.25
      uslid_stag(i,j)=uslid_stag(i,j)**0.5

      Hmxy(i,j)=((H(i,j)+H(i-1,j-1))+(H(i,j-1)+H(i-1,j)))*0.25

   end do
end do


Qslid(:,:)=ro*g*Hmxy(:,:)*pente_stag(:,:)*uslid_stag(:,:)
Qdef(:,:)=ro*g*Hmxy(:,:)*pente_stag(:,:)*vit_stag(:,:)-Qslid(:,:)
chaldef_maj(:,:,:)=0.


! On fait la moyenne de la chaleur produite sur les mailles stag

do j=2,ny-1
   do i=2,nx-1
      chalgliss_maj(i,j)=(Qslid(i,j)+Qslid(i+1,j+1))+(Qslid(i,j+1)+Qslid(i+1,j))
      chalgliss_maj(i,j)=chalgliss_maj(i,j)*0.25

      chaldef_maj(i,j,nz)=(Qdef(i,j)+Qdef(i+1,j+1))+(Qdef(i,j+1)+Qdef(i+1,j))
      chaldef_maj(i,j,nz)=chaldef_maj(i,j,nz)*0.25

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
      chalgliss_maj(i,j)=chalgliss_maj(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)

   end do
end do

!!$
!!$write(126,*)'time=',time
!!$j=41
!!$do i=20,30
!!$   write(126,'(i3,6(e14.4,1x))') i,chalgliss_maj(i,j),chaldef_maj(i,j,nz),Qdef(i,j),Qdef(i+1,j+1) &
!!$       , Qdef(i,j+1),Qdef(i+1,j)
!!$end do

! flux total a rajouter  la base
phid(:,:)=chaldef_maj(:,:,nz)+chalgliss_maj(:,:)

return
end subroutine Q_U_taub_stag

!----------------------------------------------------------------------------------
! routine qui prend la chaleur remise au socle mais sur les mailles staggered
! les noeuds stag sont ceux au milieux des mailles vitesses

subroutine Q_SIA_stag

implicit none
real,dimension(nx,ny) :: pente_stag   ! pente au carre sur les noeuds stag
real,dimension(nx,ny,nz) :: vit_stag  ! Ubar moyennee sur les noeuds stag
real,dimension(nx,ny) :: uslid_stag   ! Uslid moyennee sur les noeuds stag
real,dimension(nx,ny) :: Hmxy         ! Epaisseur moyenne sur les noeuds stag
real,dimension(nx,ny) :: Qslid        ! chaleur produite par glissement sur les noeuds stag
real,dimension(nx,ny,nz) :: Qdef      ! chaleur produite par deformation sur les noeuds stag

pente_stag = 0.
vit_stag   = 0.
uslid_stag = 0.
Hmxy  = 0.
Qslid = 0.
Qdef = 0.

do j=2,ny
   do i=2,nx

!     variables 2D
      pente_stag(i,j)=(sdx(i,j)+sdx(i,j-1))**2 + &          ! pente
                      (sdy(i,j)+sdy(i-1,j))**2

      pente_stag(i,j)=pente_stag(i,j)*0.25    ! 0.25pour la moyenne sur sdx et sdy
      pente_stag(i,j)=pente_stag(i,j)**0.5

      uslid_stag(i,j)=(ux(i,j,nz)+ux(i,j-1,nz))**2 + &        ! vitesse de bilan
                      (uy(i,j,nz)+uy(i-1,j,nz))**2
      uslid_stag(i,j)=uslid_stag(i,j)*0.25
      uslid_stag(i,j)=uslid_stag(i,j)**0.5

      Hmxy(i,j)=((H(i,j)+H(i-1,j-1))+(H(i,j-1)+H(i-1,j)))*0.25

!     en vertical : calcul de la deformation par differentiation des vitesses

      do k=1,nz

         vit_stag(i,j,k)=(ux(i,j,k)+ux(i,j-1,k))**2 + &        ! magnitude vitesse 
                         (uy(i,j,k)+uy(i-1,j,k))**2            ! a tous niveaux k

         vit_stag(i,j,k)=vit_stag(i,j,k)*0.25
         vit_stag(i,j,k)=vit_stag(i,j,k)**0.5
      end do

      Qdef(i,j,1)=0.
      do k=2,nz-1
         Qdef(i,j,k)=vit_stag(i,j,k-1)-vit_stag(i,j,k+1)            ! difference des vitesses
         Qdef(i,j,k)=ro*g*pente_stag(i,j)*e(k)*Qdef(i,j,k)/2./de    ! gamma tau
      end do


      Qdef(i,j,nz)=vit_stag(i,j,nz-1)-vit_stag(i,j,nz)               ! pour le fond differentiation
      Qdef(i,j,nz)=ro*g*pente_stag(i,j)*Qdef(i,j,k)/de               ! sur une seule maille
   end do
end do


Qslid(:,:)=ro*g*Hmxy(:,:)*pente_stag(:,:)*uslid_stag(:,:)



! On fait la moyenne de la chaleur produite sur les mailles stag

do j=2,ny-1
   do i=2,nx-1
         chalgliss_maj(i,j)=(Qslid(i,j)+Qslid(i+1,j+1))+(Qslid(i,j+1)+Qslid(i+1,j))
         chalgliss_maj(i,j)=chalgliss_maj(i,j)*0.25

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
         chalgliss_maj(i,j)=chalgliss_maj(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)

! chaleur due a la defromation
      do k=1,nz
         chaldef_maj(i,j,k)=(Qdef(i,j,k)+Qdef(i+1,j+1,k))+(Qdef(i,j+1,k)+Qdef(i+1,j,k))
         chaldef_maj(i,j,k)=chaldef_maj(i,j,k)*0.25
         chaldef_maj(i,j,k)=chaldef_maj(i,j,k)/cp(i,j,k)
      end do


   end do
end do


!!$write(126,*)'time=',time
!!$j=41
!!$do i=20,30
!!$   write(126,'(i3,6(e14.4,1x))') i,chalgliss_maj(i,j),chaldef_maj(i,j,nz),Qdef(i,j),Qdef(i+1,j+1) &
!!$       , Qdef(i,j+1),Qdef(i+1,j)
!!$end do

! flux total a rajouter  la base

phid(:,:)=chalgliss_maj(:,:)+chaldef_maj(:,:,nz)*fracq*H(:,:)*cp(:,:,nz)

return
end subroutine Q_SIA_stag

!----------------------------------------------------------------------------------
! routine qui prend la chaleur remise au socle mais sur les mailles staggered
! les noeuds stag sont ceux au milieux des mailles vitesses
! le calcul de taub tient compte des noeuds grzmx

subroutine Q_all_stag

implicit none

real, dimension(nx,ny) :: pente_stag   ! pente au carre sur les noeuds stag
real, dimension(nx,ny) :: tob_stag   ! contrainte basale
real, dimension(nx,ny) :: tox       ! contraintes sur maille mx
real, dimension(nx,ny) :: toy       ! contraintes sur maille mx
real,dimension(nx,ny,nz) :: vit_stag      ! Ubar moyennee sur les noeuds stag
real,dimension(nx,ny) :: uslid_stag    ! Uslid moyennee sur les noeuds stag
real,dimension(nx,ny) :: Hmxy          ! Epaisseur moyenne sur les noeuds stag
real,dimension(nx,ny) :: Qslid         ! chaleur produite par glissement sur les noeuds stag
real,dimension(nx,ny,nz) :: Qdef          ! chaleur produite par deformation sur les noeuds stag


where (flgzmx(:,:))
   tox(:,:)=tobmx(:,:)
elsewhere
   tox(:,:)=sdx(:,:)*hmx(:,:)*ro*g
end where

where (flgzmy(:,:))
   toy(:,:)=tobmy(:,:)
elsewhere
   toy(:,:)=sdy(:,:)*hmy(:,:)*ro*g
end where


do j=2,ny
   do i=2,nx

!     variables 2D
      pente_stag(i,j)=(sdx(i,j)+sdx(i,j-1))**2 + &          ! pente
                      (sdy(i,j)+sdy(i-1,j))**2

      pente_stag(i,j)=pente_stag(i,j)*0.25    ! 0.25pour la moyenne sur sdx et sdy
      pente_stag(i,j)=pente_stag(i,j)**0.5


      tob_stag(i,j)=(tox(i,j)+tox(i,j-1))**2 + &          ! pente
                      (toy(i,j)+toy(i-1,j))**2

      tob_stag(i,j)=tob_stag(i,j)*0.25    ! 0.25pour la moyenne sur sdx et sdy
      tob_stag(i,j)=tob_stag(i,j)**0.5

      uslid_stag(i,j)=(ux(i,j,nz)+ux(i,j-1,nz))**2 + &        ! vitesse de bilan
                      (uy(i,j,nz)+uy(i-1,j,nz))**2
      uslid_stag(i,j)=uslid_stag(i,j)*0.25
      uslid_stag(i,j)=uslid_stag(i,j)**0.5

      Hmxy(i,j)=((H(i,j)+H(i-1,j-1))+(H(i,j-1)+H(i-1,j)))*0.25

!     en vertical : calcul de la deformation par differentiation des vitesses

      do k=1,nz

         vit_stag(i,j,k)=(ux(i,j,k)+ux(i,j-1,k))**2 + &        ! magnitude vitesse 
                         (uy(i,j,k)+uy(i-1,j,k))**2            ! a tous niveaux k

         vit_stag(i,j,k)=vit_stag(i,j,k)*0.25
         vit_stag(i,j,k)=vit_stag(i,j,k)**0.5
      end do

      Qdef(i,j,1)=0.
      do k=2,nz-1
         Qdef(i,j,k)=vit_stag(i,j,k-1)-vit_stag(i,j,k+1)            ! difference des vitesses
         Qdef(i,j,k)=ro*g*pente_stag(i,j)*e(k)*Qdef(i,j,k)/2./de    ! gamma tau
      end do


      Qdef(i,j,nz)=vit_stag(i,j,nz-1)-vit_stag(i,j,nz)               ! pour le fond differentiation
      Qdef(i,j,nz)=ro*g*pente_stag(i,j)*Qdef(i,j,k)/de               ! sur une seule maille
   end do
end do


Qslid(:,:)=tob_stag*uslid_stag(:,:)



! On fait la moyenne de la chaleur produite sur les mailles stag

do j=2,ny-1
   do i=2,nx-1
         chalgliss_maj(i,j)=(Qslid(i,j)+Qslid(i+1,j+1))+(Qslid(i,j+1)+Qslid(i+1,j))
         chalgliss_maj(i,j)=chalgliss_maj(i,j)*0.25

! plus la base est loin du point de fusion moins la chaleur de glissement est prise en compte
         chalgliss_maj(i,j)=chalgliss_maj(i,j)*exp((T(i,j,nz)-tpmp(i,j,nz))*ecart_phid)

! chaleur due a la defromation
      do k=1,nz         
         chaldef_maj(i,j,k)=(Qdef(i,j,k)+Qdef(i+1,j+1,k))+(Qdef(i,j+1,k)+Qdef(i+1,j,k))
         chaldef_maj(i,j,k)=chaldef_maj(i,j,k)*0.25
         chaldef_maj(i,j,k)=chaldef_maj(i,j,k)/cp(i,j,k)
      end do


   end do
end do

!!$
!!$write(126,*)'time=',time
!!$j=41
!!$do i=20,30
!!$   write(126,'(i3,6(e14.4,1x))') i,chalgliss_maj(i,j),chaldef_maj(i,j,nz),Qdef(i,j),Qdef(i+1,j+1) &
!!$       , Qdef(i,j+1),Qdef(i+1,j)
!!$end do

! flux total a rajouter  la base
phid(:,:)=chalgliss_maj(:,:)+chaldef_maj(:,:,nz)*fracq*H(:,:)*cp(:,:,nz)
 
return
end subroutine Q_all_stag



end module ice_temp_mod
